<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Give_AmeriCloudPayments_Gateway.
 */
if ( ! class_exists( 'Give_AmeriCloudPayments_Gateway' ) ):

	class Give_AmeriCloudPayments_Gateway {
		/**
		 * Give_AmeriCloudPayments_Gateway constructor.
		 */
		function __construct() {
			add_action( 'give_gateway_americloudpayments', array( $this, 'give_process_payment' ) );
			add_action('wp_enqueue_scripts', array($this, 'load_scripts'));

			add_action( 'wp_enqueue_scripts', 'give_load_scripts' );
		}


		/**
		 * Load Scripts for AmeriCloud
		 */
		function load_scripts(){
			wp_register_script('give-americloud', GIVE_AMERICLOUDPAYMENTS_PLUGIN_URL . 'assets/js/give-americloud.js', array('jquery'), '', true);
			wp_enqueue_script('give-americloud');

			wp_register_style('give-americloud', GIVE_AMERICLOUDPAYMENTS_PLUGIN_URL . 'assets/css/give-americloud.css', array(), '0.1');
			wp_enqueue_style('give-americloud');

		}

		/**
		 * Process user submission
		 *
		 * @param $payment_data
		 */
		public function give_process_payment( $payment_data ) {
			// Validate nonce.
			give_validate_nonce( $payment_data['gateway_nonce'], 'give-gateway' );

			$payment_id = $this->give_create_payment( $payment_data );
			$data       = [];

			// Check payment.
			if ( ! empty( $payment_id ) ) {
				$data['fname']       = empty( $payment_data['user_info']['first_name'] ) ? '' : $payment_data['user_info']['first_name'];
				$data['lname']       = empty( $payment_data['user_info']['last_name'] ) ? '' : $payment_data['user_info']['last_name'];
				$data['price']       = empty( $payment_data['price'] ) ? 0 : $payment_data['price'];
				$data['card_number'] = empty( $payment_data['card_info']['card_number'] ) ? '' : $payment_data['card_info']['card_number'];
				$data['cvv2cvc2']    = empty( $payment_data['card_info']['card_cvc'] ) ? '' : $payment_data['card_info']['card_cvc'];
				$data['exp_date']    = empty( $payment_data['post_data']['card_expiry'] ) ? '' : $payment_data['post_data']['card_expiry'];

				$period = empty( $payment_data['period'] ) ? '' : $payment_data['period'];

				if ( ! empty( $period ) ) {
					switch ( $period ) {
						case 'day':
							$data['cycle'] = 'DAILY';
							break;
						case 'week':
							$data['cycle'] = 'WEEKLY';
							break;
						case 'month':
							$data['cycle'] = 'MONTHLY';
							break;
						case 'year':
							$data['cycle'] = 'ANNUALLY';
							break;
					}
					$data['next_payment_date'] = ( new DateTime( "+1 $period" ) )->format( 'm/d/Y' );
					$data['times']             = $payment_data['times'];
				}

				$data['card_address']    = $payment_data['post_data']['card_address'];
				$data['card_address_2']  = $payment_data['post_data']['card_address_2'];
				$data['card_city']       = $payment_data['post_data']['card_city'];
				$data['card_zip']        = $payment_data['post_data']['card_zip'];
				$data['card_state']      = $payment_data['post_data']['card_state'];
				$data['billing_country'] = $payment_data['post_data']['billing_country'];
				$data['user_email']      = $payment_data['post_data']['give_email'];
				$data['form_name']       = $payment_data['post_data']['give-form-title'];

				$response = $this->give_process_payment_converge( $data );

				$status_data = $this->give_process_payment_converge_get_status( $response );

				if ( $status_data['status'] ) {
					give_update_payment_status( $payment_id, 'publish' );
					give_set_payment_transaction_id( $payment_id, $status_data['txn_id'] );
					give_insert_payment_note( $payment_id, 'Result: ' . $status_data['result_message'] );

					if ( ! empty( $status_data['approval_code'] ) ) {
						give_insert_payment_note( $payment_id, 'Approval Code: ' . $status_data['approval_code'] );
					}

					if ( ! empty( $status_data['recurring'] ) && $status_data['recurring'] == true ) {

						$give_payment     = new Give_Payment( $payment_id );
						$payment_meta     = $give_payment->get_meta();
						$new_payment_meta = array( "recurring" => true );
						$merged_meta      = array_merge( $payment_meta, $new_payment_meta );

						$give_payment->update_meta( '_give_payment_meta', $merged_meta );
					}

				} else {
					//@TODO CHECK OTHER STATUS FROM PAYMENT SYSTEM AND UPDATE GIVE STATUS AND SHOW MESSAGE TO USER WITH NOTIFICATION
					//https://www.convergepay.com/converge-webapp/developer/#/converge/codes#cc-response-codes - $status_data['result_message']
					give_update_payment_status( $payment_id, 'failed' );

					if ( ! empty( $status_data['txn_id'] ) ) {
						give_set_payment_transaction_id( $payment_id, $status_data['txn_id'] );
					}
					if ( ! empty( $status_data['result_message'] ) ) {
						give_insert_payment_note( $payment_id, 'Result: ' . $status_data['result_message'] );
					}
					if ( ! empty( $status_data['approval_code'] ) ) {
						give_insert_payment_note( $payment_id, 'Approval Code: ' . $status_data['approval_code'] );
					}
				}

			} else {
				// Record the error.
				give_record_gateway_error(
					esc_html__( 'Payment Error', 'give' ),
					sprintf(
					/* translators: %s: payment data */
						esc_html__( 'Payment creation failed before sending donor to AmeriCloud Payments. Payment data: %s', 'give' ),
						json_encode( $payment_data )
					),
					$payment_id
				);
				// Problems? Send back.
				give_send_back_to_checkout( '?payment-mode=' . $payment_data['post_data']['give-gateway'] );
			}

			give_send_to_success_page();
		}


		/**
		 * Create Donate object for GIVE
		 *
		 * @param $payment_data
		 *
		 * @return bool|int
		 */
		private function give_create_payment( $payment_data ) {
			if ( $payment_data['post_data']['payment_type'] == 'give_ec' ) {
				$gateway = 'americloudpayments_ach';
			} else {
				$gateway = 'americloudpayments';
			}

			// Collect payment data.
			$insert_payment_data = array(
				'price'           => $payment_data['price'],
				'give_form_title' => $payment_data['post_data']['give-form-title'],
				'give_form_id'    => intval( $payment_data['post_data']['give-form-id'] ),
				'give_price_id'   => isset( $payment_data['post_data']['give-price-id'] ) ? $payment_data['post_data']['give-price-id'] : '',
				'date'            => $payment_data['date'],
				'user_email'      => $payment_data['user_email'],
				'purchase_key'    => $payment_data['purchase_key'],
				'currency'        => give_get_currency(),
				'user_info'       => $payment_data['user_info'],
				'status'          => 'pending',
				'gateway'         => $gateway,
			);

			return give_insert_payment( $insert_payment_data );
		}


		/**
		 * Process payment to API
		 *
		 * @param $data
		 *
		 * @internal param $live_mode
		 */
		public function give_process_payment_converge( $data ) {
			//@TODO VALIDATE INPUT DATA: $card_number, $cvv2cvc2, $exp_date
			$type       = trim( give_get_option( 'americloud_account_type' ) );
			$merchantId = trim( give_get_option( 'americloud_merchant_id' ) );
			$userId     = trim( give_get_option( 'americloud_user_id' ) );
			$pin        = trim( give_get_option( 'americloud_pin' ) );

			$type = $type == "TRUE" ? true : false;

			$PaymentProcessor = new \markroland\Converge\ConvergeApi(
				$merchantId,
				$userId,
				$pin,
				$type
			);

			$payment_data = [
				'ssl_amount'     => number_format( $data['price'], 2 ),
				'ssl_first_name' => $data['fname'],
				'ssl_last_name'  => $data['lname'],
			];

			if ( ! $type ) {
				$payment_data['ssl_cvv2cvc2_indicator'] = 1;
				$payment_data['ssl_card_present']       = "N";
			}

			$payment_data['ssl_card_number'] = $data['card_number'];
			$payment_data['ssl_cvv2cvc2']    = $data['cvv2cvc2'];
			$payment_data['ssl_exp_date']    = $data['exp_date'];
			$payment_data['ssl_city']        = $data['card_city'];
			$payment_data['ssl_state']       = $data['card_state'];
			$payment_data['ssl_avs_zip']     = $data['card_zip'];
			$payment_data['ssl_avs_address'] = $data['card_address'];
			$payment_data['ssl_address2']    = $data['card_address_2'];
			$payment_data['ssl_country']     = $data['billing_country'];
			$payment_data['ssl_email']       = $data['user_email'];
			$payment_data['ssl_description'] = $data['form_name'];

			if ( $data['cycle'] ) {
				// Submit a purchase with CreditCard & recurring option
				$payment_data['ssl_billing_cycle']     = $data['cycle'];
				$payment_data['ssl_next_payment_date'] = $data['next_payment_date'];

				if ( $data['times'] > 0 ) {
					$payment_data['ssl_total_installments'] = $data['times'];
					$response                               = $PaymentProcessor->ccaddinstall( $payment_data );
				} else {
					$response = $PaymentProcessor->ccaddrecurring( $payment_data );
				}

				// Sybmit Recurring Payment
				if ( ( $response['ssl_result_message'] == 'SUCCESS' )
				     && ( ! empty( $response['ssl_recurring_id'] ) )
				     && ( $response['ssl_result'] == 0 )
				) {

					$sale_data = array(
						"ssl_recurring_id" => $response["ssl_recurring_id"]
					);

					$response = $PaymentProcessor->ccrecurringsale( $sale_data );

				} elseif ( ( $response['ssl_result_message'] == 'SUCCESS' )
				           && ( ! empty( $response['ssl_installment_id'] ) )
				           && ( $response['ssl_result'] == 0 )
				) {

					$sale_data = array(
						"ssl_installment_id" => $response["ssl_installment_id"]
					);

					$response = $PaymentProcessor->ccinstallsale( $sale_data );
				}

			} else {
				// Submit a purchase with CreditCard
				$response = $PaymentProcessor->ccsale( $payment_data );
			}

			return $response;
		}

		/**
		 * Get status of payment
		 *
		 * @param $response - response from ConvergePay API
		 *
		 * @return array
		 */
		private function give_process_payment_converge_get_status( $response ) {
			$status     = false;
			$data_array = array();

			if ( is_array( $response ) ) {
				if ( ( ! empty( $response['ssl_result_message'] ) )
				     && ( $response['ssl_result_message'] == 'APPROVAL' )
				     && ( ! empty( $response['ssl_txn_id'] ) )
				     && ( ! empty( $response['ssl_approval_code'] ) )
				) {
					$status = true;
				}

				if ( ( ! empty( $response['ssl_result_message'] ) )
				     && ( $response['ssl_result_message'] == 'SUCCESS' )
				     && ( ! empty( $response['ssl_txn_id'] ) )
				     && ( $response['ssl_result'] == 0 )
				) {
					$status    = true;
					$recurring = true;
				}
			}

			return array(
				'status'         => $status,
				'result_message' => ( is_array( $response ) && ( ! empty( $response['ssl_result_message'] ) ) ) ? $response['ssl_result_message'] : '',
				'txn_id'         => ( is_array( $response ) && ( ! empty( $response['ssl_txn_id'] ) ) ) ? $response['ssl_txn_id'] : '',
				'approval_code'  => ( is_array( $response ) && ( ! empty( $response['ssl_approval_code'] ) ) ) ? $response['ssl_approval_code'] : '',
				"recurring"      => $recurring
			);
		}

		public function check_admin_keys() {
			$settings = array(
				'type'       => trim( give_get_option( 'americloud_account_type' ) ),
				'merchantID' => trim( give_get_option( 'americloud_merchant_id' ) ),
				'userI'      => trim( give_get_option( 'americloud_user_id' ) ),
				'pin'        => trim( give_get_option( 'americloud_pin' ) )
			);

			if ( in_array( '', $settings ) ) {
				return false;
			} else {
				return true;
			}
		}

	}

	new Give_AmeriCloudPayments_Gateway();

endif;